/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ext.ensembl;

import jalview.datamodel.AlignmentI;

import java.io.BufferedReader;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Arrays;
import java.util.List;

import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

/**
 * A client for the Ensembl lookup REST endpoint, used to find the gene
 * identifier given a gene, transcript or protein identifier.
 * 
 * @author gmcarstairs
 */
public class EnsemblLookup extends EnsemblRestClient
{
  /**
   * Default constructor (to use rest.ensembl.org)
   */
  public EnsemblLookup()
  {
    super();
  }

  /**
   * Constructor given the target domain to fetch data from
   * 
   * @param
   */
  public EnsemblLookup(String d)
  {
    super(d);
  }

  @Override
  public String getDbName()
  {
    return "ENSEMBL";
  }

  @Override
  public AlignmentI getSequenceRecords(String queries) throws Exception
  {
    return null;
  }

  @Override
  protected URL getUrl(List<String> ids) throws MalformedURLException
  {
    String identifier = ids.get(0);
    return getUrl(identifier, null);
  }

  /**
   * Gets the url for lookup of the given identifier, optionally with objectType
   * also specified in the request
   * 
   * @param identifier
   * @param objectType
   * @return
   */
  protected URL getUrl(String identifier, String objectType)
  {
    String url = getDomain() + "/lookup/id/" + identifier
            + CONTENT_TYPE_JSON;
    if (objectType != null)
    {
      url += "&" + OBJECT_TYPE + "=" + objectType;
    }

    try
    {
      return new URL(url);
    } catch (MalformedURLException e)
    {
      return null;
    }
  }

  @Override
  protected boolean useGetRequest()
  {
    return true;
  }

  @Override
  protected String getRequestMimeType(boolean multipleIds)
  {
    return "application/json";
  }

  @Override
  protected String getResponseMimeType()
  {
    return "application/json";
  }

  /**
   * Returns the gene id related to the given identifier, which may be for a
   * gene, transcript or protein
   * 
   * @param identifier
   * @return
   */
  public String getGeneId(String identifier)
  {
    return getGeneId(identifier, null);
  }

  /**
   * Returns the gene id related to the given identifier (which may be for a
   * gene, transcript or protein)
   * 
   * @param identifier
   * @param objectType
   * @return
   */
  public String getGeneId(String identifier, String objectType)
  {
    List<String> ids = Arrays.asList(new String[] { identifier });

    BufferedReader br = null;
    try
    {
      URL url = getUrl(identifier, objectType);
      if (url != null)
      {
        br = getHttpResponse(url, ids);
      }
      return br == null ? null : parseResponse(br);
    } catch (IOException e)
    {
      // ignore
      return null;
    } finally
    {
      if (br != null)
      {
        try
        {
          br.close();
        } catch (IOException e)
        {
          // ignore
        }
      }
    }
  }

  /**
   * Parses the JSON response and returns the gene identifier, or null if not
   * found. If the returned object_type is Gene, returns the id, if Transcript
   * returns the Parent. If it is Translation (peptide identifier), then the
   * Parent is the transcript identifier, so we redo the search with this value.
   * 
   * @param br
   * @return
   * @throws IOException
   */
  protected String parseResponse(BufferedReader br) throws IOException
  {
    String geneId = null;
    JSONParser jp = new JSONParser();
    try
    {
      JSONObject val = (JSONObject) jp.parse(br);
      String type = val.get(OBJECT_TYPE).toString();
      if (OBJECT_TYPE_GENE.equalsIgnoreCase(type))
      {
        // got the gene - just returns its id
        geneId = val.get(ID).toString();
      }
      else if (OBJECT_TYPE_TRANSCRIPT.equalsIgnoreCase(type))
      {
        // got the transcript - return its (Gene) Parent
        geneId = val.get(PARENT).toString();
      }
      else if (OBJECT_TYPE_TRANSLATION.equalsIgnoreCase(type))
      {
        // got the protein - get its Parent, restricted to type Transcript
        String transcriptId = val.get(PARENT).toString();
        geneId = getGeneId(transcriptId, OBJECT_TYPE_TRANSCRIPT);
      }
    } catch (ParseException e)
    {
      // ignore
    }
    return geneId;
  }

}
